package com.company.grouptableavg.web.screens.order;

import com.haulmont.chile.core.model.MetaPropertyPath;
import com.haulmont.cuba.core.global.DataManager;
import com.haulmont.cuba.core.global.Metadata;
import com.haulmont.cuba.gui.components.AggregationInfo;
import com.haulmont.cuba.gui.components.GroupTable;
import com.haulmont.cuba.gui.components.Table;
import com.haulmont.cuba.gui.data.aggregation.AggregationStrategy;
import com.haulmont.cuba.gui.screen.*;
import com.company.grouptableavg.entity.Order;

import javax.inject.Inject;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collection;
import java.util.List;

import static java.math.BigDecimal.ZERO;

@UiController("grouptableavg_Order.browse")
@UiDescriptor("order-browse.xml")
@LookupComponent("ordersTable")
@LoadDataBeforeShow
public class OrderBrowse extends StandardLookup<Order> {
    @Inject
    private DataManager dataManager;
    @Inject
    private GroupTable<Order> ordersTable;
    @Inject
    private Metadata metadata;

    @Subscribe
    public void onInit(InitEvent event) {
        if (dataManager.load(Order.class).list().size() == 0) {
            Order order = dataManager.create(Order.class);
            order.setCustomer("cus1");
            order.setAmount(BigDecimal.valueOf(1000));
            order.setCost(BigDecimal.valueOf(500));
            order.setMargin(BigDecimal.valueOf(500));
            order.setMarginRate(BigDecimal.valueOf(50));
            dataManager.commit(order);
            order = dataManager.create(Order.class);
            order.setCustomer("cus2");
            order.setAmount(BigDecimal.valueOf(2000));
            order.setCost(BigDecimal.valueOf(500));
            order.setMargin(BigDecimal.valueOf(1500));
            order.setMarginRate(BigDecimal.valueOf(75));
            dataManager.commit(order);
        }
        MetaPropertyPath metaPropertyPath = metadata.getClassNN(Order.class).getPropertyPath("marginRate");
        AggregationInfo info = new AggregationInfo();
        info.setStrategy(new MarginRateAggregationStrategy());
        Table.Column column = ordersTable.getColumn("marginRate");
        column.setAggregation(info);
    }

    public static class MarginRateAggregationStrategy implements AggregationStrategy<Order, BigDecimal> {
        @Override
        public BigDecimal aggregate(Collection<Order> orders) {
//            BigDecimal totalAmount = ZERO;
//            BigDecimal totalMargin = ZERO;
//            for (List<Order> list : orders) {
//                for (Order order : list) {
//                    totalAmount = totalAmount.add(list.stream().map(Order::getAmount).reduce(ZERO, BigDecimal::add));
//                    totalMargin = totalMargin.add(list.stream().map(Order::getMargin).reduce(ZERO, BigDecimal::add));
//                }
//            }
//            BigDecimal totalAmount = orders.stream().flatMap(Collection::stream).map(Order::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
//            BigDecimal totalMargin = orders.stream().flatMap(Collection::stream).map(Order::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

            BigDecimal totalAmount = orders.stream().map(Order::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal totalMargin = orders.stream().map(Order::getMargin).reduce(BigDecimal.ZERO, BigDecimal::add);

            BigDecimal marginRate = totalMargin.multiply(BigDecimal.valueOf(100)).divide(totalAmount, 2, RoundingMode.HALF_UP);
            return marginRate;
        }

        @Override
        public Class<BigDecimal> getResultClass() {
            return BigDecimal.class;
        }
    }

}